#define MODULE "main"

#include "acpi/RSDT.h"
#include "assert.h"
#include "bootboot.h"
#include "config.h"
#include "cpu/CPU.h"
#include "gdt/GDT.h"
#include "init/Init.h"
#include "init/InitRD.h"
#include "interrupts/IDT.h"
#include "interrupts/Install.h"
#include "interrupts/Interrupts.h"
#include "io/PIC.h"
#include "io/Serial.h"
#include "log/Address.h"
#include "log/Log.h"
#include "memory/KernelHeap.h"
#include "memory/KernelMemoryManager.h"
#include "memory/Memory.h"
#include "memory/MemoryMap.h"
#include "memory/RangeAllocator.h"
#include "panic/hang.h"
#include "power/reboot.h"
#include "render/BBRenderer.h"
#include "render/Draw.h"
#include "render/TextRenderer.h"
#include "scheduling/PIT.h"
#include "std/stdio.h"
#include "std/stdlib.h"
#include "std/string.h"

extern BOOTBOOT bootboot;

extern "C" void _start()
{
    Init::check_magic();
    Init::disable_smp(); // Put all other cores except the bootstrap one in an infinite loop
    Init::early_init();

    kinfoln("Starting Moon %d.%d%s", MOON_MAJOR, MOON_MINOR, MOON_SUFFIX);

    CPU::log_cpu_information();

    Memory::walk_memory_map();

    GDT::load();

    kinfoln("Loaded GDT");

    Interrupts::install();

    IDT::load();

    kinfoln("Loaded IDT");

    PIC::remap();
    PIC::enable_master(0b11111100); // enable keyboard and PIT
    PIC::enable_slave(0b11111111);

    kinfoln("Prepared PIC");

    PIT::initialize(1000); // 1000 times per second, 1 time per millisecond

    kinfoln("Prepared PIT");

    Interrupts::enable();

    kinfoln("Interrupts enabled");

    ACPI::SDTHeader* rootSDT = ACPI::GetRSDTOrXSDT();
    bool isXSDT = ACPI::IsXSDT(rootSDT);
    if (!ACPI::ValidateRSDTOrXSDT(rootSDT)) kerrorln("Invalid %s", isXSDT ? "XSDT" : "RSDT");

    sleep(2500);
    reboot();

    while (1) halt();
}